<?php
namespace Financeiro\Logic\ControleBancario\LancamentoemContaCorrente;
use \Financeiro\Entities\Movdebbanc;
use \Financeiro\Entities\Movcredbanc;
use \Financeiro\Entities\Movbancario;
use \Geral\Util\DateUtil;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use \Contabilidade\Logic\Util;


class LancamentoemContaCorrente
{
    const IDTIPOCRED = 'C';
    const IDTIPODEBI = 'D';
    const IDENTRADA = 'Entrada';
    const IDSAIDA = 'Saida';

    public static function deleteMovibancario($cdreducont, $nrmovbanc)
    {
        try {

            $movbancario = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc(
                (\Geral\Util\Ambiente::getNrorg()), $cdreducont, $nrmovbanc
            );
            $movbancario->delete();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function deleteMovidebi($cdreducont, $nrmovideb)
    {
        try {
            /** @var $movdebbanc Movdebbanc */
            $movdebbanc = Movdebbanc::findOneByNrorgAndCdreducontAndNrmovidebi(
                (\Geral\Util\Ambiente::getNrorg()), $cdreducont, $nrmovideb
            );
            $dtmovi = $movdebbanc->getDtmovidebi();
            $vrmovi = $movdebbanc->getVrmovidebi();
            $movdebbanc->delete();

            \Setforms2\Shared\Registry::getEntityManager()->flush();

            return array('DTMOVI' => $dtmovi, 'VRMOVI' => $vrmovi);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function deleteMovicred($cdreducont, $nrmovicred)
    {
        try {
            /** @var $movcredbanc Movcredbanc */
            $movcredbanc = Movcredbanc::findOneByNrorgAndCdreducontAndNrmovicred(
                (\Geral\Util\Ambiente::getNrorg()), $cdreducont, $nrmovicred
            );
            $dtmovi = $movcredbanc->getDtmovicred();
            $vrmovi = $movcredbanc->getVrmovicred();
            $movcredbanc->delete();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            return array('DTMOVI' => $dtmovi, 'VRMOVI' => $vrmovi);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateMovibancario($cdreducont, $nrmovbanc, $dshismbanc)
    {
        try {
            $movbancario = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc(
                (\Geral\Util\Ambiente::getNrorg()), $cdreducont, $nrmovbanc
            );
            $movbancario->setDshismbanc($dshismbanc);

            $movbancario->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateMovidebi($cdreducont, $nrmovdebi, $dsmovidebi)
    {
        try {
            /** @var $movdebbanc Movdebbanc */
            $movdebbanc = Movdebbanc::findOneByNrorgAndCdreducontAndNrmovidebi(
                \Geral\Util\Ambiente::getNrorg(), $cdreducont, $nrmovdebi
            );
            $movdebbanc->setDsmovidebi($dsmovidebi);

            $movdebbanc->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            return array('DTMOVI' => $movdebbanc->getDtmovidebi(), 'VRMOVI' => $movdebbanc->getVrmovidebi());

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateMovicred($cdreducont, $nrmovicred, $dsmovicred)
    {
        try {
            /** @var $movcredbanc Movcredbanc */
            $movcredbanc = Movcredbanc::findOneByNrorgAndCdreducontAndNrmovicred(
                (\Geral\Util\Ambiente::getNrorg()), $cdreducont, $nrmovicred
            );
            $movcredbanc->setDsmovicred($dsmovicred);

            $movcredbanc->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            return array('DTMOVI' => $movcredbanc->getDtmovicred(), 'VRMOVI' => $movcredbanc->getVrmovicred());

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function insereMovicred(
        $cdreducont, $nrmovicred, $cdclassfina, $cdempresa, $cdfilial,
        $idmovfluxo, $nrmovbanc, $cdtipo, $dtmovicred, $dsmovicred, $vrmovicred,
        $cdemplanctb, $lanccont, $cdfililanca, $cdcliente, $cdforneced
    )
    {
        try {
            FluxodeCaixa::atualizaFluxo(
                $cdempresa, $cdfilial, $cdclassfina, $dtmovicred, $vrmovicred, $idmovfluxo, self::IDENTRADA
            );


            $movcredbanc = new Movcredbanc();
            $movcredbanc->setCdreducont($cdreducont);
            $movcredbanc->setNrmovicred($nrmovicred);
            $movcredbanc->setCdclassfina($cdclassfina);
            $movcredbanc->setCdempresa($cdempresa);
            $movcredbanc->setCdfilial($cdfilial);
            $movcredbanc->setDtmovfluxo(self::formataData($dtmovicred));
            $movcredbanc->setDtinclusao(DateUtil::getDataAtual());
            $movcredbanc->setIdmovfluxo($idmovfluxo);
            $movcredbanc->setNrmovbanc($nrmovbanc);
            $movcredbanc->setCdtpcredbanc($cdtipo);
            $movcredbanc->setDtmovicred(self::formataData($dtmovicred));
            $movcredbanc->setDsmovicred($dsmovicred);
            $movcredbanc->setVrmovicred($vrmovicred);
            $movcredbanc->setCdectbmcred($cdemplanctb);
            $movcredbanc->setNrlctbmcred($lanccont);
            $movcredbanc->setCdfililancre($cdfililanca);
            $movcredbanc->setCdcliente($cdcliente);
            $movcredbanc->setCdforneced($cdforneced);
            $movcredbanc->setIdlancomcontrec('N');
            $movcredbanc->setNrorg(\Geral\Util\Ambiente::getNrorg());

            $movcredbanc->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function insereMovidebi(
        $cdreducont, $nrmovidebi, $cdclassfina, $cdempresa, $cdfilial,
        $idmovfluxo, $nrmovbanc, $cdtipo, $dtmovidebi, $dsmovidebi, $vrmovidebi,
        $cdemplanctb, $lanccont, $cdfililanca, $cdcliente, $cdforneced
    )
    {
        try {
            FluxodeCaixa::atualizaFluxo(
                $cdempresa, $cdfilial, $cdclassfina, $dtmovidebi, $vrmovidebi, $idmovfluxo, self::IDSAIDA
            );

            $movdebbanc = new Movdebbanc();
            $movdebbanc->setCdreducont($cdreducont);
            $movdebbanc->setNrmovidebi($nrmovidebi);
            $movdebbanc->setCdclassfina($cdclassfina);
            $movdebbanc->setCdempresa($cdempresa);
            $movdebbanc->setCdfilial($cdfilial);
            $movdebbanc->setDtmovfluxo(self::formataData($dtmovidebi));
            $movdebbanc->setDtinclusao(DateUtil::getDataAtual());
            $movdebbanc->setIdmovfluxo($idmovfluxo);
            $movdebbanc->setNrmovbanc($nrmovbanc);
            $movdebbanc->setCdtpdebibanc($cdtipo);
            $movdebbanc->setDtmovidebi(self::formataData($dtmovidebi));
            $movdebbanc->setDsmovidebi($dsmovidebi);
            $movdebbanc->setVrmovidebi($vrmovidebi);
            $movdebbanc->setCdectbmdeb($cdemplanctb);
            $movdebbanc->setNrlctbmdeb($lanccont);
            $movdebbanc->setCdfililandeb($cdfililanca);
            $movdebbanc->setCdcliente($cdcliente);
            $movdebbanc->setCdforneced($cdforneced);
            $movdebbanc->setIdlancomcontpag('N');
            $movdebbanc->setNrorg(\Geral\Util\Ambiente::getNrorg());

            $movdebbanc->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function insereMovibancario($cdreducont, $nrmovbanc, $dtmovi, $Idmovi, $nrdocmbanc, $vrmovi, $idstconext, $dsmovi)
    {
        try {
            $movbancario = new Movbancario();
            $movbancario->setCdreducont($cdreducont);
            $movbancario->setNrmovbanc($nrmovbanc);
            $movbancario->setDtmovbanc(self::formataData($dtmovi));
            $movbancario->setDtinclusao(DateUtil::getDataAtual());
            $movbancario->setIdmovbanc($Idmovi);
            $movbancario->setNrdocmbanc($nrdocmbanc);
            $movbancario->setVrmovbanc($vrmovi);
            $movbancario->setNrorg(\Geral\Util\Ambiente::getNrorg());
            $movbancario->setDshismbanc($dsmovi);
            $movbancario->setIdorigmbanc($Idmovi);
            $movbancario->setIdstconext($idstconext);
            $movbancario->setIdstatusctbbanc('P');

            $movbancario->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function formataData($data)
    {
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    public static function contabilizaLancamento($cdreducont, $nrmovi, $idtipo)
    {
        try {
            $nrlancctb = '';
            $procedure = new Procedure('CTB_BCO_LANC');
            $procedure->addParam(new Param('P_CDREDUCONT', Param::PARAM_INPUT, $cdreducont, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRMOVI', Param::PARAM_INPUT, $nrmovi, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, $nrlancctb, Param::PARAM_TYPE_STR, 12));
            $procedure->call();
            $nrlancctb = $procedure->call();
            return $nrlancctb['P_NRLANCCTB'];


        } catch (\Exception $e) {
            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e, true);
        }

    }

}